@@ -102,8 +102,11 @@ class Ao3Extractor(Extractor):
     def _pagination(self, path, needle='<li id="work_'):
         while True:
             page = self.request(self.root + path).text
+
             yield from text.extract_iter(page, needle, '"')
-            path = text.extr(page, '<a rel="next" href="', '"')
+
+            path = (text.extr(page, '<a rel="next" href="', '"') or
+                    text.extr(page, '<li class="next"><a href="', '"'))
             if not path:
                 return
             path = text.unescape(path)
@@ -205,8 +208,7 @@ class Ao3WorkExtractor(Ao3Extractor):
         }
         data["language"] = util.code_to_language(data["lang"])
 
-        series = data["series"]
-        if series:
+        if series := data["series"]:
             extr = text.extract_from(series)
             data["series"] = {
                 "prev" : extr(' class="previous" href="/works/', '"'),
