@@ -6,7 +6,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for https://kemono.su/"""
+"""Extractors for https://kemono.cr/"""
 
 from .common import Extractor, Message
 from .. import text, util, exception
@@ -14,7 +14,8 @@ from ..cache import cache, memcache
 import itertools
 import json
 
-BASE_PATTERN = r"(?:https?://)?(?:www\.|beta\.)?(kemono|coomer)\.(su|party)"
+BASE_PATTERN = (r"(?:https?://)?(?:www\.|beta\.)?"
+                r"(kemono|coomer)\.(cr|s[tu]|party)")
 USER_PATTERN = BASE_PATTERN + r"/([^/?#]+)/user/([^/?#]+)"
 HASH_PATTERN = r"/[0-9a-f]{2}/[0-9a-f]{2}/([0-9a-f]{64})"
 
@@ -22,17 +23,17 @@ HASH_PATTERN = r"/[0-9a-f]{2}/[0-9a-f]{2}/([0-9a-f]{64})"
 class KemonoExtractor(Extractor):
     """Base class for kemono extractors"""
     category = "kemono"
-    root = "https://kemono.su"
+    root = "https://kemono.cr"
     directory_fmt = ("{category}", "{service}", "{user}")
     filename_fmt = "{id}_{title[:180]}_{num:>02}_{filename[:180]}.{extension}"
     archive_fmt = "{service}_{user}_{id}_{num}"
-    cookies_domain = ".kemono.su"
+    cookies_domain = ".kemono.cr"
 
     def __init__(self, match):
-        tld = match[2]
-        self.category = domain = match[1]
-        self.root = text.root_from_url(match[0])
-        self.cookies_domain = f".{domain}.{tld}"
+        if match[1] == "coomer":
+            self.category = "coomer"
+            self.root = "https://coomer.st"
+            self.cookies_domain = ".coomer.st"
         Extractor.__init__(self, match)
 
     def _init(self):
@@ -44,7 +45,7 @@ class KemonoExtractor(Extractor):
         self.revisions_reverse = order[0] in ("r", "a") if order else False
 
         self._find_inline = util.re(
-            r'src="(?:https?://(?:kemono|coomer)\.su)?(/inline/[^"]+'
+            r'src="(?:https?://(?:kemono\.cr|coomer\.st))?(/inline/[^"]+'
             r'|/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{64}\.[^"]+)').findall
         self._json_dumps = json.JSONEncoder(
             ensure_ascii=False, check_circular=False,
@@ -101,19 +102,15 @@ class KemonoExtractor(Extractor):
                 post["username"] = creator["name"]
 
             if comments:
-                try:
-                    post["comments"] = self.api.creator_post_comments(
-                        service, creator_id, post["id"])
-                except exception.HttpError:
+                post["comments"] = cmts = self.api.creator_post_comments(
+                    service, creator_id, post["id"])
+                if not isinstance(cmts, list):
+                    self.log.debug("%s/%s: %s", creator_id, post["id"], cmts)
                     post["comments"] = ()
             if dms is not None:
                 if dms is True:
                     dms = self.api.creator_dms(
                         post["service"], post["user"])
-                    try:
-                        dms = dms["props"]["dms"]
-                    except Exception:
-                        dms = ()
                 post["dms"] = dms
             if announcements is not None:
                 if announcements is True:
@@ -154,7 +151,8 @@ class KemonoExtractor(Extractor):
                     file["extension"] = ext
                 elif ext == "txt" and file["extension"] != "txt":
                     file["_http_validate"] = _validate
-                elif ext in exts_archive:
+                elif ext in exts_archive or \
+                        ext == "bin" and file["extension"] in exts_archive:
                     file["type"] = "archive"
                     if archives:
                         try:
@@ -244,16 +242,15 @@ class KemonoExtractor(Extractor):
     def _revisions_post(self, post):
         post["revision_id"] = 0
 
-        try:
-            revs = self.api.creator_post_revisions(
-                post["service"], post["user"], post["id"])
-        except exception.HttpError:
+        revs = self.api.creator_post_revisions(
+            post["service"], post["user"], post["id"])
+        if not revs:
             post["revision_hash"] = self._revision_hash(post)
             post["revision_index"] = 1
             post["revision_count"] = 1
             return (post,)
-        revs.insert(0, post)
 
+        revs.insert(0, post)
         for rev in revs:
             rev["revision_hash"] = self._revision_hash(rev)
 
@@ -312,10 +309,10 @@ def _validate(response):
 
 
 class KemonoUserExtractor(KemonoExtractor):
-    """Extractor for all posts from a kemono.su user listing"""
+    """Extractor for all posts from a kemono.cr user listing"""
     subcategory = "user"
     pattern = USER_PATTERN + r"/?(?:\?([^#]+))?(?:$|\?|#)"
-    example = "https://kemono.su/SERVICE/user/12345"
+    example = "https://kemono.cr/SERVICE/user/12345"
 
     def __init__(self, match):
         self.subcategory = match[3]
@@ -324,32 +321,21 @@ class KemonoUserExtractor(KemonoExtractor):
     def posts(self):
         _, _, service, creator_id, query = self.groups
         params = text.parse_query(query)
-        tag = params.get("tag")
 
-        endpoint = self.config("endpoint")
-        if endpoint == "legacy+":
-            endpoint = self._posts_legacy_plus
-        elif endpoint == "legacy" or tag:
-            endpoint = self.api.creator_posts_legacy
+        if self.config("endpoint") in ("posts+", "legacy+"):
+            endpoint = self.api.creator_posts_expand
         else:
             endpoint = self.api.creator_posts
 
         return endpoint(service, creator_id,
-                        params.get("o"), params.get("q"), tag)
-
-    def _posts_legacy_plus(self, service, creator_id,
-                           offset=0, query=None, tags=None):
-        for post in self.api.creator_posts_legacy(
-                service, creator_id, offset, query, tags):
-            yield self.api.creator_post(
-                service, creator_id, post["id"])["post"]
+                        params.get("o"), params.get("q"), params.get("tag"))
 
 
 class KemonoPostsExtractor(KemonoExtractor):
-    """Extractor for kemono.su post listings"""
+    """Extractor for kemono.cr post listings"""
     subcategory = "posts"
     pattern = BASE_PATTERN + r"/posts()()(?:/?\?([^#]+))?"
-    example = "https://kemono.su/posts"
+    example = "https://kemono.cr/posts"
 
     def posts(self):
         params = text.parse_query(self.groups[4])
@@ -358,10 +344,10 @@ class KemonoPostsExtractor(KemonoExtractor):
 
 
 class KemonoPostExtractor(KemonoExtractor):
-    """Extractor for a single kemono.su post"""
+    """Extractor for a single kemono.cr post"""
     subcategory = "post"
     pattern = USER_PATTERN + r"/post/([^/?#]+)(/revisions?(?:/(\d*))?)?"
-    example = "https://kemono.su/SERVICE/user/12345/post/12345"
+    example = "https://kemono.cr/SERVICE/user/12345/post/12345"
 
     def __init__(self, match):
         self.subcategory = match[3]
@@ -387,14 +373,14 @@ class KemonoPostExtractor(KemonoExtractor):
 
 
 class KemonoDiscordExtractor(KemonoExtractor):
-    """Extractor for kemono.su discord servers"""
+    """Extractor for kemono.cr discord servers"""
     subcategory = "discord"
     directory_fmt = ("{category}", "discord",
                      "{server_id} {server}", "{channel_id} {channel}")
     filename_fmt = "{id}_{num:>02}_{filename}.{extension}"
     archive_fmt = "discord_{server_id}_{id}_{num}"
     pattern = BASE_PATTERN + r"/discord/server/(\d+)[/#](?:channel/)?(\d+)"
-    example = "https://kemono.su/discord/server/12345/12345"
+    example = "https://kemono.cr/discord/server/12345/12345"
 
     def items(self):
         _, _, server_id, channel_id = self.groups
@@ -422,8 +408,7 @@ class KemonoDiscordExtractor(KemonoExtractor):
         find_hash = util.re(HASH_PATTERN).match
 
         posts = self.api.discord_channel(channel_id)
-        max_posts = self.config("max-posts")
-        if max_posts:
+        if max_posts := self.config("max-posts"):
             posts = itertools.islice(posts, max_posts)
 
         for post in posts:
@@ -461,7 +446,7 @@ class KemonoDiscordExtractor(KemonoExtractor):
 class KemonoDiscordServerExtractor(KemonoExtractor):
     subcategory = "discord-server"
     pattern = BASE_PATTERN + r"/discord/server/(\d+)$"
-    example = "https://kemono.su/discord/server/12345"
+    example = "https://kemono.cr/discord/server/12345"
 
     def items(self):
         server_id = self.groups[2]
@@ -486,10 +471,10 @@ def discord_server_info(extr, server_id):
 
 
 class KemonoFavoriteExtractor(KemonoExtractor):
-    """Extractor for kemono.su favorites"""
+    """Extractor for kemono.cr favorites"""
     subcategory = "favorite"
     pattern = BASE_PATTERN + r"/(?:account/)?favorites()()(?:/?\?([^#]+))?"
-    example = "https://kemono.su/account/favorites/artists"
+    example = "https://kemono.cr/account/favorites/artists"
 
     def items(self):
         self.login()
@@ -537,7 +522,7 @@ class KemonoArtistsExtractor(KemonoExtractor):
     """Extractor for kemono artists"""
     subcategory = "artists"
     pattern = BASE_PATTERN + r"/artists(?:\?([^#]+))?"
-    example = "https://kemono.su/artists"
+    example = "https://kemono.cr/artists"
 
     def items(self):
         params = text.parse_query(self.groups[2])
@@ -572,12 +557,13 @@ class KemonoArtistsExtractor(KemonoExtractor):
 class KemonoAPI():
     """Interface for the Kemono API v1.1.0
 
-    https://kemono.su/documentation/api
+    https://kemono.cr/documentation/api
     """
 
     def __init__(self, extractor):
         self.extractor = extractor
         self.root = extractor.root + "/api/v1"
+        self.headers = {"Accept": "text/css"}
 
     def posts(self, offset=0, query=None, tags=None):
         endpoint = "/posts"
@@ -589,20 +575,21 @@ class KemonoAPI():
         return self._call(endpoint)
 
     def creators(self):
-        endpoint = "/creators.txt"
+        endpoint = "/creators"
         return self._call(endpoint)
 
     def creator_posts(self, service, creator_id,
                       offset=0, query=None, tags=None):
-        endpoint = f"/{service}/user/{creator_id}"
-        params = {"q": query, "tag": tags, "o": offset}
+        endpoint = f"/{service}/user/{creator_id}/posts"
+        params = {"o": offset, "tag": tags, "q": query}
         return self._pagination(endpoint, params, 50)
 
-    def creator_posts_legacy(self, service, creator_id,
+    def creator_posts_expand(self, service, creator_id,
                              offset=0, query=None, tags=None):
-        endpoint = f"/{service}/user/{creator_id}/posts-legacy"
-        params = {"o": offset, "tag": tags, "q": query}
-        return self._pagination(endpoint, params, 50, "results")
+        for post in self.creator_posts(
+                service, creator_id, offset, query, tags):
+            yield self.creator_post(
+                service, creator_id, post["id"])["post"]
 
     def creator_announcements(self, service, creator_id):
         endpoint = f"/{service}/user/{creator_id}/announcements"
@@ -622,11 +609,11 @@ class KemonoAPI():
 
     def creator_post_comments(self, service, creator_id, post_id):
         endpoint = f"/{service}/user/{creator_id}/post/{post_id}/comments"
-        return self._call(endpoint)
+        return self._call(endpoint, fatal=False)
 
     def creator_post_revisions(self, service, creator_id, post_id):
         endpoint = f"/{service}/user/{creator_id}/post/{post_id}/revisions"
-        return self._call(endpoint)
+        return self._call(endpoint, fatal=False)
 
     def creator_profile(self, service, creator_id):
         endpoint = f"/{service}/user/{creator_id}/profile"
@@ -657,19 +644,24 @@ class KemonoAPI():
         params = {"type": type}
         return self._call(endpoint, params)
 
-    def _call(self, endpoint, params=None):
-        url = self.root + endpoint
-        response = self.extractor.request(url, params=params)
-        return response.json()
+    def _call(self, endpoint, params=None, headers=None, fatal=True):
+        if headers is None:
+            headers = self.headers
+        else:
+            headers = {**self.headers, **headers}
 
-    def _pagination(self, endpoint, params, batch=50, key=False):
+        return self.extractor.request_json(
+            f"{self.root}{endpoint}", params=params, headers=headers,
+            encoding="utf-8", fatal=fatal)
+
+    def _pagination(self, endpoint, params, batch=50, key=None):
         offset = text.parse_int(params.get("o"))
         params["o"] = offset - offset % batch
 
         while True:
             data = self._call(endpoint, params)
 
-            if key:
+            if key is not None:
                 data = data.get(key)
             if not data:
                 return
