@@ -320,20 +320,17 @@ def dump_response(response, fp, headers=False, content=True, hide_auth=True):
         res_headers = response.headers.copy()
 
         if hide_auth:
-            authorization = req_headers.get("Authorization")
-            if authorization:
+            if authorization := req_headers.get("Authorization"):
                 atype, sep, _ = str(authorization).partition(" ")
                 req_headers["Authorization"] = f"{atype} ***" if sep else "***"
 
-            cookie = req_headers.get("Cookie")
-            if cookie:
+            if cookie := req_headers.get("Cookie"):
                 req_headers["Cookie"] = ";".join(
                     c.partition("=")[0] + "=***"
                     for c in cookie.split(";")
                 )
 
-            set_cookie = res_headers.get("Set-Cookie")
-            if set_cookie:
+            if set_cookie := res_headers.get("Set-Cookie"):
                 res_headers["Set-Cookie"] = re(r"(^|, )([^ =]+)=[^,;]*").sub(
                     r"\1\2=***", set_cookie)
 
@@ -377,14 +374,11 @@ def extract_headers(response):
     headers = response.headers
     data = dict(headers)
 
-    hcd = headers.get("content-disposition")
-    if hcd:
-        name = text.extr(hcd, 'filename="', '"')
-        if name:
+    if hcd := headers.get("content-disposition"):
+        if name := text.extr(hcd, 'filename="', '"'):
             text.nameext_from_url(name, data)
 
-    hlm = headers.get("last-modified")
-    if hlm:
+    if hlm := headers.get("last-modified"):
         data["date"] = datetime.datetime(*parsedate_tz(hlm)[:6])
 
     return data
@@ -518,15 +512,15 @@ def cookiestxt_store(fp, cookies):
             value = cookie.value
 
         domain = cookie.domain
-        fp.write("\t".join((
-            domain,
-            "TRUE" if domain and domain[0] == "." else "FALSE",
-            cookie.path,
-            "TRUE" if cookie.secure else "FALSE",
-            "0" if cookie.expires is None else str(cookie.expires),
-            name,
-            value + "\n",
-        )))
+        fp.write(
+            f"{domain}\t"
+            f"{'TRUE' if domain and domain[0] == '.' else 'FALSE'}\t"
+            f"{cookie.path}\t"
+            f"{'TRUE' if cookie.secure else 'FALSE'}\t"
+            f"{'0' if cookie.expires is None else str(cookie.expires)}\t"
+            f"{name}\t"
+            f"{value}\n"
+        )
 
 
 def code_to_language(code, default=None):
@@ -993,16 +987,21 @@ def build_proxy_map(proxies, log=None):
     if isinstance(proxies, str):
         if "://" not in proxies:
             proxies = "http://" + proxies.lstrip("/")
-        return {"http": proxies, "https": proxies}
-
-    if isinstance(proxies, dict):
+        proxies = {"http": proxies, "https": proxies}
+    elif isinstance(proxies, dict):
         for scheme, proxy in proxies.items():
             if "://" not in proxy:
                 proxies[scheme] = "http://" + proxy.lstrip("/")
-        return proxies
+    else:
+        proxies = None
 
     if log is not None:
-        log.warning("invalid proxy specifier: %s", proxies)
+        if proxies is None:
+            log.warning("Invalid proxy specifier: %r", proxies)
+        else:
+            log.debug("Proxy Map: %s", proxies)
+
+    return proxies
 
 
 def build_predicate(predicates):
